#include <assert.h>

#include <aterm2.h>
#include <deprecated.h>
#include "list.h"

/*{{{  typedefs */

typedef struct ATerm _Modules;
typedef struct ATerm _ModuleList;

/*}}}  */

/*{{{  void initListApi(void) */

void initListApi(void)
{
  init_list_dict();
}

/*}}}  */

/*{{{  term conversion functions */

/*{{{  Modules ModulesFromTerm(ATerm t) */

Modules ModulesFromTerm(ATerm t)
{
  return (Modules)t;
}

/*}}}  */
/*{{{  ATerm ModulesToTerm(Modules arg) */

ATerm ModulesToTerm(Modules arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ModuleList ModuleListFromTerm(ATerm t) */

ModuleList ModuleListFromTerm(ATerm t)
{
  return (ModuleList)t;
}

/*}}}  */
/*{{{  ATerm ModuleListToTerm(ModuleList arg) */

ATerm ModuleListToTerm(ModuleList arg)
{
  return (ATerm)arg;
}

/*}}}  */

/*}}}  */
/*{{{  constructors */

/*{{{  Modules makeModulesList(Module first, Modules next) */

Modules makeModulesList(Module first, Modules next)
{
  return (Modules)(ATerm)ATmakeAppl1(afun0, (ATerm)ATinsert((ATermList)next, (ATerm)first));
}

/*}}}  */
/*{{{  Modules makeModulesBody(Module first, Modules next) */

Modules makeModulesBody(Module first, Modules next)
{
  return (Modules)(ATerm)ATinsert((ATermList)next, (ATerm)first);
}

/*}}}  */
/*{{{  ModuleList makeModuleListModules(Modules modules) */

ModuleList makeModuleListModules(Modules modules)
{
  return (ModuleList)(ATerm)ATmakeAppl1(afun1, (ATerm)modules);
}

/*}}}  */

/*}}}  */
/*{{{  equality functions */

ATbool isEqualModules(Modules arg0, Modules arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualModuleList(ModuleList arg0, ModuleList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

/*}}}  */
/*{{{  Modules accessors */

/*{{{  ATbool isValidModules(Modules arg) */

ATbool isValidModules(Modules arg)
{
  if (isModulesList(arg)) {
    return ATtrue;
  }
  else if (isModulesBody(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isModulesList(Modules arg) */

inline ATbool isModulesList(Modules arg)
{
  if (ATgetType((ATerm)arg) != AT_APPL) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternModulesList, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool isModulesBody(Modules arg) */

inline ATbool isModulesBody(Modules arg)
{
  if (ATgetType((ATerm)arg) != AT_LIST) {
    return ATfalse;
  }
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternModulesBody, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool hasModulesFirst(Modules arg) */

ATbool hasModulesFirst(Modules arg)
{
  if (isModulesList(arg)) {
    return ATtrue;
  }
  else if (isModulesBody(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  Module getModulesFirst(Modules arg) */

Module getModulesFirst(Modules arg)
{
  if (isModulesList(arg)) {
    return (Module)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 0));
  }
  else 
    return (Module)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  Modules setModulesFirst(Modules arg, Module first) */

Modules setModulesFirst(Modules arg, Module first)
{
  if (isModulesList(arg)) {
    return (Modules)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 0), (ATerm)first, 0), 0);
  }
  else if (isModulesBody(arg)) {
    return (Modules)ATreplace((ATermList)arg, (ATerm)first, 0);
  }

  ATabort("Modules has no First: %t\n", arg);
  return (Modules)NULL;
}

/*}}}  */
/*{{{  ATbool hasModulesNext(Modules arg) */

ATbool hasModulesNext(Modules arg)
{
  if (isModulesList(arg)) {
    return ATtrue;
  }
  else if (isModulesBody(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  Modules getModulesNext(Modules arg) */

Modules getModulesNext(Modules arg)
{
  if (isModulesList(arg)) {
    return (Modules)ATgetNext((ATermList)ATgetArgument((ATermAppl)arg, 0));
  }
  else 
    return (Modules)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  Modules setModulesNext(Modules arg, Modules next) */

Modules setModulesNext(Modules arg, Modules next)
{
  if (isModulesList(arg)) {
    return (Modules)ATsetArgument((ATermAppl)arg, (ATerm)ATreplaceTail((ATermList)ATgetArgument((ATermAppl)arg, 0), (ATermList)next, 1), 0);
  }
  else if (isModulesBody(arg)) {
    return (Modules)ATreplaceTail((ATermList)arg, (ATermList)next, 1);
  }

  ATabort("Modules has no Next: %t\n", arg);
  return (Modules)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ModuleList accessors */

/*{{{  ATbool isValidModuleList(ModuleList arg) */

ATbool isValidModuleList(ModuleList arg)
{
  if (isModuleListModules(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isModuleListModules(ModuleList arg) */

inline ATbool isModuleListModules(ModuleList arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternModuleListModules, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool hasModuleListModules(ModuleList arg) */

ATbool hasModuleListModules(ModuleList arg)
{
  if (isModuleListModules(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  Modules getModuleListModules(ModuleList arg) */

Modules getModuleListModules(ModuleList arg)
{
  
    return (Modules)ATgetArgument((ATermAppl)arg, 0);
}

/*}}}  */
/*{{{  ModuleList setModuleListModules(ModuleList arg, Modules modules) */

ModuleList setModuleListModules(ModuleList arg, Modules modules)
{
  if (isModuleListModules(arg)) {
    return (ModuleList)ATsetArgument((ATermAppl)arg, (ATerm)modules, 0);
  }

  ATabort("ModuleList has no Modules: %t\n", arg);
  return (ModuleList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  sort visitors */

/*{{{  Modules visitModules(Modules arg, Module (*acceptFirst)(Module)) */

Modules visitModules(Modules arg, Module (*acceptFirst)(Module))
{
  if (isModulesList(arg)) {
    return makeModulesList(
        acceptFirst ? acceptFirst(getModulesFirst(arg)) : getModulesFirst(arg),
        visitModules(getModulesNext(arg), acceptFirst));
  }
  if (isModulesBody(arg)) {
    return makeModulesBody(
        acceptFirst ? acceptFirst(getModulesFirst(arg)) : getModulesFirst(arg),
        visitModules(getModulesNext(arg), acceptFirst));
  }
  ATabort("not a Modules: %t\n", arg);
  return (Modules)NULL;
}

/*}}}  */
/*{{{  ModuleList visitModuleList(ModuleList arg, Modules (*acceptModules)(Modules)) */

ModuleList visitModuleList(ModuleList arg, Modules (*acceptModules)(Modules))
{
  if (isModuleListModules(arg)) {
    return makeModuleListModules(
        acceptModules ? acceptModules(getModuleListModules(arg)) : getModuleListModules(arg));
  }
  ATabort("not a ModuleList: %t\n", arg);
  return (ModuleList)NULL;
}

/*}}}  */

/*}}}  */
